//
// Zilog Developer Studio Intel hexadecimal file format to Texas Instruments TI86p file converter.
// based on the ZDS Hex to TI-86 Raw Hex converter (Beta).
// File: zdsto86p
// Version: 1.0
// Date: September 13, 2003
// Author: Todd Martin Miller
//
// Usage Notes:
// The output file is the same as a TI86 GraphLink file (version 2.0).
// The Comment will be a date and time string from the computer the program is executed upon.
// The program name will be the first 8 characters or characters until the first space characters
// of the input file name.
//

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>


#define MAX_INPUT_FILE_LINES		100			// Maximum number of assembly code lines in file.
fstream in, out;
void writeHexidecimal(int value);				// Prototypes
char *max (int v1, int v2);


int main(int argc, char *argv[])
{
	char infile_name[501] = "\0", 
		outfile_name[501] = "\0",
		buffer[MAX_INPUT_FILE_LINES][101],
		write_buffer = '\0',
		throwAway[11];

	system("CLS");
	cout<<"- ZDS Intel Hex to TI86p Converter (1.0) -\n"<<endl;

	if(argc==1)		//if no input file specified
	{
		cout<<"Proper usage:\n"<<endl;
		cout<<"\tzdsto86p [filename(.hex)]"<<endl;
		getch();
		exit(0);
	}

	int length = strlen(argv[1]);
	strcpy(infile_name, argv[1]);
	in.open(infile_name, ios::in | ios::nocreate);
	if(!in)
	{
		cout<<"Error opening "<<infile_name<<"!"<<endl;
		exit(0);
	}

	cout<<"Converting "<<infile_name<<endl<<endl;

	for(int i=0; i<length-4; i++)	//minus 4 to remove .hex extension
		outfile_name[i] = infile_name[i];
	strcat(outfile_name, ".86p");

	cout<<"Outputting "<<outfile_name<<endl<<endl;

	out.open(outfile_name, ios::out | ios::binary);	//ios::noreplase
	
	// Begin File Header.
	// Write the Identifying string for the TI86.
	out<<"**TI86**";

	out<<(unsigned char) 26;
	out<<(unsigned char) 10;
	out<<(unsigned char) 0;

	// Write a 41 character comment with the date and time in it.
	time_t now;
	now = time(NULL);
	char *theTime = ctime(&now);
	int char_count = 0;

	char *saveTime = theTime;
	while((unsigned char) *theTime++ != (unsigned char) 10)
	{
		char_count++;
	}
	*--theTime = (unsigned char) 0;
	// Write comment.
	out<<saveTime;
	while(char_count < 42) {
		out<<(unsigned char) 0;
		char_count++;
	}

	// Read the code from the input file.
	int lineNumber = 0;
	int byte_count = 0;
	while(!in.eof())					// Read file throwing away address and checksum bytes
	{		
		in.getline(throwAway,10,'\n');					// Remove code addresses
		memset(buffer[lineNumber],0,100);
		in.getline(buffer[lineNumber], 101, '\n');
		int lineLength = strlen(buffer[lineNumber]);
		if(lineLength > 0) {
			char_count =  lineLength-2;					// Remove check sums
			buffer[lineNumber][lineLength-1] = 0;
			buffer[lineNumber][lineLength-2] = 0;
			byte_count = byte_count + char_count;
		}
		lineNumber++;
	}
	int checkSum = 0;
	// Calculate total length of output file in bytes.
	int lengthOfHeaders = 72;
	int lineDelimiters = 0;
	for(i=0;i<lineNumber;i++)
	{
		if(buffer[i][0] != 0)
			lineDelimiters++;
	}
	lineDelimiters--;									// last line does not get a 0xd6 delimiter.
	int codeStartDelimiter = 1;							// a zero at front of code.
	int checkSumLength = 2;
	int asmPrgmHeaderLength = 8;					    // includes 0xd6 line delimiter.
	int fileLength =	lengthOfHeaders + 
						codeStartDelimiter + 
						asmPrgmHeaderLength +
						byte_count + 
						lineDelimiters +
						checkSumLength;	

	// Size of file beyond this header; File Size minus 0x38.
	writeHexidecimal(fileLength-0x38);


	// Begin Check Sum
	out<<(unsigned char) 0x0c;
	out<<(unsigned char) 0;
	checkSum = checkSum + 0xC;
	// End

	// Begin Program variable Header at 0x39 (first byte numbered 0)
	// Size of the code and checksum in data part.
	int codeSize =	codeStartDelimiter +
					asmPrgmHeaderLength +
					byte_count +
					lineDelimiters +
					checkSumLength;
	writeHexidecimal(codeSize);
	checkSum = checkSum + ((codeSize & 0xFF00)/256) + (codeSize & 0xFF);

	out<<(unsigned char) 0x12;							// variable type
	out<<(unsigned char) 0x08;							// variable name length
	checkSum = checkSum + 0x12 + 0x08;
	
	// Write Program Name
	char *ProgramNameStart;
	char *ProgramNameStart1 = strrchr(outfile_name,92);
	char *ProgramNameStart2 = strrchr(outfile_name,'/');
	char *ProgramNameStart3 = strrchr(outfile_name,']');
	char *ProgramNameStart4 = strrchr(outfile_name,':');
	if((int)(ProgramNameStart1) == 0 && (int)(ProgramNameStart2) == 0 && (int)(ProgramNameStart3) ==0 &&
		(int)(ProgramNameStart4) == 0)
	{
		ProgramNameStart = &outfile_name[0];
	} else {
		ProgramNameStart = max((int) (ProgramNameStart1), (int) (ProgramNameStart2));
		ProgramNameStart = max((int) (ProgramNameStart), (int) (ProgramNameStart3));
		ProgramNameStart = max((int) (ProgramNameStart), (int) (ProgramNameStart4));
		ProgramNameStart++;
	}

	i = 0;
	while(*ProgramNameStart != (char) (' ') && i < 8)
		outfile_name[i++] = *ProgramNameStart++;

	if(i < 8 || outfile_name[i] != (unsigned char) (0))
		outfile_name[i++] = (unsigned char) (0);

	out<<(unsigned char ) outfile_name[i];			// variable name
	for(i=0;i < 8; i++)
		checkSum = checkSum + outfile_name[i];

	// End Program Variable Header.


	// Begin Data Section at 0x45
	// Size of the code and checksum in data part beginning at 0x49.
	writeHexidecimal(codeSize);
	checkSum = checkSum + ((codeSize & 0xFF00)/256) + (codeSize & 0xFF); 

	// Size of the code alone which begins at 0x49.
	writeHexidecimal(codeSize-2);
	checkSum = checkSum + ((codeSize-2 & 0xFF00)/256) + (codeSize-2 & 0xFF);; 

	// code start delimiter
	out<<(unsigned char) 0;

	// TI Assembly Language Command
	out<<"AsmPrgm";
	checkSum = checkSum + 'A' + 's' + 'm' + 'P' + 'r' + 'g' + 'm';

	out<<(unsigned char) 0xd6;							// line delimiter
	checkSum = checkSum + 0xd6;

	// Now write the program code.
	for(int j=0; j<lineDelimiters+1;j++)
	{
		i = 0;
		while(buffer[j][i] != 0) {
			out<< (unsigned char) buffer[j][i];
			checkSum = checkSum + buffer[j][i];
			i++;
		}
		if(j<(lineDelimiters)) {
			out<<(unsigned char) 0xd6;					// line delimiter
			checkSum = checkSum + 0xd6;
		}
	}

	// Write Check Sum.
	out<<(unsigned char) (int) (checkSum & 0xFF);
	out<<(unsigned char) (int) ((checkSum & 0xFF00)/256);
	// End Data Section.


	out.close();
	in.close();

	cout<<"Success!"<<endl;

	return (0);
}


void writeHexidecimal(int value)
{
	// The Texas Instruments file requires 16 bit words in hexademical be written
	// Least significant byte first and Most significant byte second.  For example,
	// if 0x0063 needs to be written to the file then it is written as 0x6300.
	unsigned char temp1 = (value & 0xFF00)/256;
	unsigned char temp2 = value & 0x00FF;
	out<<temp2;
	out<<temp1;
	return;
}

char * max(int v1, int v2)
{
	if(v1 > v2)
		return (char *) (v1);
	else
		return (char *) (v2);
}